/*
 * fexec.c
 *
 * Demonstrates fork/exec
 *
 * Bernhard Rabe
 *
 * 03.11.2004
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>

int main(int argc,char *argv[])
{
    pid_t pid1;
    int status,ret;
    
    if(argc<3)
    {
        printf("Usage: %s executable programname\n",argv[0]);
        exit(1);
    }
    
    pid1=fork();
    
    if(pid1<0)
    { 
        perror("error at fork\n"); 
        exit(1); 
    }

    if(pid1==0)
    { 
        printf("I'm the child: %d of %d\n",getpid(),getppid());
        printf("%d starting %s\n",getpid(),argv[1]);
        
        execl(argv[1],argv[2],0);
        
        ret=errno;
        fprintf(stderr,"%d error starting %s: %s\n",getpid(),argv[1],strerror(ret));
        exit(42);
    }
        
    printf("I'm the parent: %d of %d\n",getpid(),pid1);
    
    if (waitpid(pid1, &status, 0) < 0)
    {
        int errval=errno;
	    fprintf(stderr,"%d waitpid had not performed:%s",getpid(),strerror(errval));
	    exit(1);
    }

    if (WIFEXITED(status))
    {
        printf ("%d exit status %d\n", getpid(), WEXITSTATUS(status));
    }
    else
    {
	    printf ("%d process has terminated abnormal\n",getpid());
    }
    return 0;
}
